// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef CHECKSUM_HPP
#define CHECKSUM_HPP

#include <stdint.h>

//-----------------------------------------------------------------------------
// Checksum
//-----------------------------------------------------------------------------

/**
 * @brief Class that provides checksum calculation.
 */
template<typename Iterator>
class Checksum
{
public:
    Checksum(
            const Iterator &body_begin,
            const Iterator &body_end
    );

    // TODO common checksum
    // The <typename Message> must have a method which returns an array of words
    // this way the checksum calculator can be used with any ICMP, TCP, UDP...
    // uint16_t compute( const Message &message ) const
    // {
    //      message.get_word_array() -> uint16_t *word_array
    //
    //
    uint16_t compute(
            uint8_t type,
            uint8_t code,
            uint16_t identifier,
            uint16_t sequence_number
    ) const;

private:
    Iterator BodyBegin;
    Iterator BodyEnd;

};

template<typename Iterator>
    Checksum<Iterator>::Checksum(
            const Iterator &body_begin,
            const Iterator &body_end
    ) :
        BodyBegin( body_begin ),
        BodyEnd( body_end )
    {
    }

template<typename Iterator>
    uint16_t Checksum<Iterator>::compute(
            uint8_t type,
            uint8_t code,
            uint16_t identifier,
            uint16_t sequence_number
    ) const
    {
        uint32_t sum = (type << 8) + code + identifier + sequence_number;

        Iterator body_iter = BodyBegin;
        while ( body_iter != BodyEnd )
        {
            sum += (static_cast<uint8_t>( *body_iter++ ) << 8);
            if ( body_iter != BodyEnd )
                sum += static_cast<uint8_t>( *body_iter++ );
        }

        sum = (sum >> 16) + (sum & 0xFFFF);
        sum += (sum >> 16);

        uint16_t checksum = static_cast<uint16_t>( ~sum );

        return checksum;
    }

#endif // CHECKSUM_HPP
