/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "icmp/icmppacket.h"
#include <iostream>

//-----------------------------------------------------------------------------
// IcmpPacket
//-----------------------------------------------------------------------------

/**
 * @brief Default constructor.
 */
IcmpPacket::IcmpPacket()
{
}

/**
 * @brief Destructor.
 */
IcmpPacket::~IcmpPacket()
{
}


std::string IcmpPacket::return_code_to_string( const IcmpPacket::ReadReturnCode &code )
{
    switch (code)
    {
    case IcmpPacket::ReadReturnCode_OK: return "ok"; break;
    case IcmpPacket::ReadReturnCode_UNSPECIFIED: return "return code not set yet!"; break;
    case IcmpPacket::ReadReturnCode_FAIL: return "failed for unspecified reason!"; break;
    case IcmpPacket::ReadReturnCode_NOT_ENOUGH_DATA: return "not enough data!"; break;
    case IcmpPacket::ReadReturnCode_BAD_STREAM: return "bad stream!"; break;
    case IcmpPacket::ReadReturnCode_INVALID_SIZE: return "invalid data size!"; break;
    case IcmpPacket::ReadReturnCode_UNKNOWN_PROTOCOL: return "unknown protocol, expect ICMPv4/6!"; break;
    }

    // if nothing matched
    return "unknown return code!";
}

/**
 * @brief create packet from data in @a is;  calls IcmpPacket::read
 *
 * @param is input stream with hopefully sufficient data
 *
 * @returns stream with less data; failbit is set if read returns other than ReadReturnCode_OK
 */
std::istream& operator>>(
        std::istream &is,
        IcmpPacket &packet
)
{
    IcmpPacket::ReadReturnCode return_code = packet.read(is);
    if ( return_code != IcmpPacket::ReadReturnCode_OK )
        is.setstate( std::ios::failbit );

    return is;
}


std::ostream& operator<<(
        std::ostream& os,
        const IcmpPacket& packet
)
{
    packet.write( os );
    return os;
}
